<?php

class DispletRetsIdxUsers{
	public function register_user_ajax(){
		// Check nonce for security
		check_ajax_referer('displet_register_user_nonce');

		if (isset($_POST['action']) && $_POST['action'] == 'displet_user_registration_request'){
			if (!empty($_POST['name']) && !empty($_POST['email'])) {
				if (is_email($_POST['email'])) {
					$user_registration_response = self::register_user($_POST['email'], $_POST['name'], $_POST['phone'], $_POST['url'], $_POST['upstream_url'], $_POST['last_hash']);
					echo $user_registration_response;
				}
				else{
					echo 'Please enter a valid email address.';
				}
			}
			else{
				echo 'Please complete the required fields.';
			}
			die();
		}

		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}

	public function register_user($email, $name, $phone = '', $url = '', $upstream_url = '', $last_hash = '') {
		// Create WP User
		$random_password = wp_generate_password($length=12, $include_standard_special_chars=false);
		$user_id = wp_create_user($email, $random_password, $email);
		if (is_wp_error($user_id)) {
			return $user_id->get_error_message();
		}
		$user_info = get_userdata($user_id);

		// Send email
		DispletRetsIdxEmails::send_new_user_registration($email, $name, $phone, $url, $user_info->user_login, $random_password);

		// Login user
		$creds = array(
			'user_login' => get_userdata($user_id)->user_login,
			'user_password' => $random_password,
			'remember' => true
		);
		$user = wp_signon($creds, false);

		// Assign user meta
		update_user_meta($user_id, 'displet_lead', true);
		update_user_meta($user_id, 'nickname', $name);
		update_user_meta($user_id, 'displet_phone', $phone);
		update_user_meta($user_id, 'displet_registration_url', $url);
		update_user_meta($user_id, 'displet_registration_upstream_url', $upstream_url);
		update_user_meta($user_id, 'displet_last_hash', $last_hash);

		// Update user role
		wp_update_user(array(
			'ID' => $user_id,
			'role' => 'displet_user'
		));

		// Send to Zapier
		$zap_user = array(
			'email' => $email,
			'name' => $name,
			'phone' => $phone,
			'registered_at' => $url,
			'came_from' => $upstream_url
		);
		self::send_to_zapier($zap_user, 'registration');

		return 'Successful Registration';
	}

	public function login_user_ajax() {
		// Check nonce for security
		check_ajax_referer('displet_login_user_nonce');

		if (isset($_POST['action']) && $_POST['action'] == 'displetretsidx_user_signon'){
			if (!empty($_POST['email']) && !empty($_POST['password'])) {
				$creds = array();
				$creds['user_login'] = $_POST['email'];
				$creds['user_password'] = $_POST['password'];
				$creds['remember'] = true;
				$user = wp_signon($creds, false);
				if (is_wp_error($user)) {
					echo $user->get_error_message();
				}
				else{
					echo 'Successful Login';
				}
			}
			else {
				echo 'Please complete the required fields.';
			}
			die();
		}
		
		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}

	public function check_user_ajax() {
		// Check nonce for security
		check_ajax_referer('displet_check_user_nonce');

		if (isset($_POST['action']) && $_POST['action'] == 'displet_user_check_request'){
			if (!empty($_POST['email'])) {
				$user_exists = self::check_user($_POST['email']);
				if ($user_exists) {
					echo 'User Exists and Logged In';
				}
			}
			else {
				echo 'There is no email account associated with this user.';
			}
			die();
		}
		
		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}

	public function check_user($email){
		$user_id = username_exists($email);
		if (!empty($user_id)) {
			wp_set_auth_cookie($user_id, true);
			return true;
		}
		else{
			return false;
		}
	}

	public function check_login_ajax(){
		// Check nonce for security
		check_ajax_referer('displet_check_login_nonce');

		if (is_user_logged_in()) {
			echo 'Successful Login';
		}
		else{
			echo 'User Not Logged In';
		}
		die();
	}

	public function update_last_login($user_login, $user){
		update_user_meta($user->ID, 'displet_last_login', date('Y/m/d \a\t g:ia', current_time('timestamp')));
	}

	public function update_viewed_searches(){
		// Check nonce for security
		check_ajax_referer('displet_update_searches_nonce');
		
		if (isset($_POST['action']) && $_POST['action'] == 'displet_update_searches' && !empty($_POST['last_hash'])){
			$user_id = get_current_user_id();
			if ($user_id != 0 && current_user_can('displet_save_searches') && !current_user_can('manage_options')) {
				$user_searches = get_user_meta($user_id, 'displet_user_hashes', true);
				$user_searches[] = $_POST['last_hash'];
				update_user_meta($user_id, 'displet_user_hashes', $user_searches);
			}
		}
		die();
	}

	public function update_viewed_properties($url = '', $address = '', $price = '', $zip = '', $sq_ft = ''){
		$user_id = get_current_user_id();
		if ($user_id != 0 && current_user_can('displet_save_properties') && !current_user_can('manage_options')) {
			$user_searches = get_user_meta($user_id, 'displet_user_properties', true);
			$user_searches[] = array(
				'url' => $url,
				'address' => $address,
				'price' => $price,
				'zip' => $zip,
				'sq_ft' => $sq_ft
			);
			update_user_meta($user_id, 'displet_user_properties', $user_searches);
		}
	}

	public function delete_users(){
		// Check nonce for security
		check_ajax_referer('displet_delete_users_nonce');

		if (isset($_POST['action']) && $_POST['action'] == 'displet_delete_users_request'){
			if (!empty($_POST['displet_users'])) {
				$error_occurred = false;
				foreach ($_POST['displet_users'] as $user){
					$deleted[] = wp_delete_user($user);
				}
				foreach ($deleted as $was_deleted) {
					if (!$was_deleted){
						$error_occurred = true;
					}
				}
				if ($error_occurred){
					echo 'There was an error processing your request.';
				}
				else{
					echo 'Succesful Deletion';
				}
			}
			die();
		}
	}

	public function save_property(){
		// Check nonce for security
		check_ajax_referer('displet_save_property_nonce');

		if (isset($_POST['action']) && $_POST['action'] == 'displet_save_property_request'){
			if (current_user_can('displet_save_properties') && !current_user_can('manage_options')) {
				if (!empty($_POST['type'])) {
					$displetretsidx_option = DispletRetsIdxSettings::get_option();
					$user_id = get_current_user_id();
					if ($user_id != 0) {
						$base = get_permalink($displetretsidx_option['property_details_page_id']);
						$url = str_replace($base, '', $_POST['url']);
						$saved_properties = get_user_meta($user_id, 'displet_saved_properties', true);
						$saved_properties[] = array(
							'sysid' => $_POST['sysid'],
							//'address' => $_POST['address'],
							//'city' => $_POST['city'],
							//'state' => $_POST['state'],
							//'zip' => $_POST['zip'],
							'url' => $url,
							'type' => $_POST['type'],
							'rating' => $_POST['rating'],
							'message' => $_POST['message'],
						);
						update_user_meta($user_id, 'displet_saved_properties', $saved_properties);
						echo 'This property has been saved.';
					}
					else{
						echo 'You are not logged in. Please sign in or create an account and try again.';
					}
				}
				else{
					echo 'Please select Save as favorite, Save as possibility, or Just save notes.';
				}
			}
			else{
				echo 'The current user does not have the capability to save properties.';
			}
			die();
		}
		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}

	public function delete_saved_properties(){
		// Check nonce for security
		check_ajax_referer('displet_delete_properties_nonce');

		if (isset($_POST['action']) && $_POST['action'] == 'displet_delete_properties_request'){
			if (!empty($_POST['displet_properties'])) {
				$user_id = get_current_user_id();
				$saved_properties = get_user_meta($user_id, 'displet_saved_properties');
				$delete_properties = $_POST['displet_properties'];
				foreach ($delete_properties as $property_id){
					unset($saved_properties[0][$property_id]);
				}
				$saved_properties[0] = array_values($saved_properties[0]);
				update_user_meta($user_id, 'displet_saved_properties', $saved_properties[0]);
				echo 'Succesful Deletion';
			}
			else{
				echo 'No property was selected for deletion.';
			}
			die();
		}
		else {
			echo 'There was an error processing your request. Please try again.';
		}
		die();
	}

	public function save_search(){
		// Check nonce for security
		check_ajax_referer('displet_save_search_nonce');

		if (isset($_POST['action']) && $_POST['action'] == 'displet_save_search_request'){
			if (!empty($_POST['hash'])) {
				$user_id = get_current_user_id();
				if ($user_id != 0) {
					if (current_user_can('displet_save_searches') && !current_user_can('manage_options')) {
						$saved_searches = get_user_meta($user_id, 'displet_saved_searches', true);
						$saved_searches[] = array(
							'hash' => $_POST['hash']
						);
						update_user_meta($user_id, 'displet_saved_searches', $saved_searches);
						echo 'Your search has been saved successfully.';
					}
					else{
						echo 'The current user does not have the capability to save properties.';
					}
				}
				else{
					echo 'You are not logged in. Please sign in or create an account and try again.';
				}
			}
			else{
				echo 'Please perform a search to save.';
			}
			die();
		}
		
		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}

	public function delete_saved_searches(){
		// Check nonce for security
		check_ajax_referer('displet_delete_searches_nonce');

		if (isset($_POST['action']) && $_POST['action'] == 'displet_delete_searches_request'){
			if (!empty($_POST['displet_searches'])) {
				$user_id = get_current_user_id();
				$saved_searches = get_user_meta($user_id, 'displet_saved_searches');
				$delete_searches = $_POST['displet_searches'];
				foreach ($delete_searches as $search_id){
					unset($saved_searches[0][$search_id]);
				}
				$saved_searches[0] = array_values($saved_searches[0]);
				update_user_meta($user_id, 'displet_saved_searches', $saved_searches[0]);
				echo 'Succesful Deletion';
			}
			else{
				echo 'No property was selected for deletion.';
			}
			die();
		}
		else {
			echo 'There was an error processing your request. Please try again.';
		}
		die();
	}

	public function set_referral_cookie(){
		if (empty($_COOKIE['displetretsidx_referring_site'])) {
        	setcookie('displetretsidx_referring_site', $_SERVER['HTTP_REFERER'], 0, '/');
		}
	}

	public function add_custom_user_role(){
		// Making changes in this function requires updating the user_role_version
		$displetretsidx_option = DispletRetsIdxSettings::get_option();
		if (empty($displetretsidx_option['user_role_version']) || $displetretsidx_option['user_role_version'] < 1){
			remove_role('displet_user');
			$capabilities = array(
				'read' => true,
				'displet_save_properties' => true,
				'displet_save_searches' => true
			);
			add_role('displet_user', 'Real Estate Search User', $capabilities);
			$displetretsidx_option['user_role_version'] = 1;
			update_option('displet_rets_idx_options', $displetretsidx_option);
		}
	}

	public function add_saved_properties_menu(){
		if (!current_user_can('manage_options')) {
			add_menu_page('Saved Properties', 'Saved Properties', 'displet_save_properties', 'displetretsidx_saved_properties', array('DispletRetsIdxUsers', 'add_saved_properties_page'), plugins_url('displet-retsidx-beta/css/images/saved-properties-icon.png'), 71);
		}
	}

	public function add_saved_properties_page(){
		require_once dirname(__FILE__) . '/users/displet-saved-properties-page.php';
	}

	public function add_saved_searches_menu(){
		if (!current_user_can('manage_options')) {
			add_menu_page('Saved Searches', 'Saved Searches', 'displet_save_searches', 'displetretsidx_saved_searches', array('DispletRetsIdxUsers', 'add_saved_searches_page'), plugins_url('displet-retsidx-beta/css/images/saved-searches-icon.png'), 72);
		}
	}

	public function add_saved_searches_page(){
		require_once dirname(__FILE__) . '/users/displet-saved-searches-page.php';
	}

	public function add_search_properties_to_toolbar($wp_admin_bar){
		if (current_user_can('displet_save_properties') && !current_user_can('manage_options')) {
			$displetretsidx_option = DispletRetsIdxSettings::get_option();
			$href = get_permalink($displetretsidx_option['search_results_page_id']);
    		$wp_admin_bar->add_node(array(
    			'id' => 'displetretsidx_search_properties_toolbar',
    			'title' => 'Search Properties',
    			'href' => $href
    		));
    		$wp_admin_bar->add_node(array(
    			'id' => 'displetretsidx_search_properties_toolbar_SUB',
    			'title' => 'Search Properties',
    			'href' => $href,
    			'parent' => 'site-name'
    		));
		}
	}

	public function add_saved_properties_to_toolbar($wp_admin_bar){
		if (current_user_can('displet_save_properties') && !current_user_can('manage_options')) {
    		$wp_admin_bar->add_node(array(
    			'id' => 'displetretsidx_saved_properties_toolbar',
    			'title' => 'Saved Properties',
    			'href' => admin_url('admin.php?page=displetretsidx_saved_properties')
    		));
    		$wp_admin_bar->add_node(array(
    			'id' => 'displetretsidx_saved_properties_toolbar_sub',
    			'title' => 'Saved Properties',
    			'href' => admin_url('admin.php?page=displetretsidx_saved_properties'),
    			'parent' => 'site-name'
    		));
		}
	}

	public function add_saved_searches_to_toolbar($wp_admin_bar){
		if (current_user_can('displet_save_searches') && !current_user_can('manage_options')) {
    		$wp_admin_bar->add_node(array(
    			'id' => 'displetretsidx_saved_searches_toolbar',
    			'title' => 'Saved Searches',
    			'href' => admin_url('admin.php?page=displetretsidx_saved_searches')
    		));
    		$wp_admin_bar->add_node(array(
    			'id' => 'displetretsidx_saved_searches_toolbar_sub',
    			'title' => 'Saved Searches',
    			'href' => admin_url('admin.php?page=displetretsidx_saved_searches'),
    			'parent' => 'site-name'
    		));
		}
	}

	public function clean_up_user_toolbar($wp_admin_bar){
		if (current_user_can('displet_save_properties') && !current_user_can('manage_options')) {
    		$wp_admin_bar->remove_node('wp-logo');
    		$wp_admin_bar->remove_node('search');
		}
	}

	public function clean_up_user_dashboard(){
		if (current_user_can('displet_save_properties') && !current_user_can('manage_options')) {
			remove_meta_box('dashboard_primary', 'dashboard', 'side');
			remove_meta_box('dashboard_secondary', 'dashboard', 'side');
		}
	}

	public function send_to_zapier($user_data, $zap){
		$displetretsidx_option = DispletRetsIdxSettings::get_option();
		if (!empty($displetretsidx_option['zapier_' . $zap . '_url'])) {
			$request = array(
				'user-agent' => 'Displet',
				'body' => $user_data,
				'referer' => site_url()
			);
			$urls = explode("\n", $displetretsidx_option['zapier_' . $zap . '_url']);
			foreach ($urls as $url) {
				$post_url = trim($url);
				if (!empty($post_url)) {
					wp_remote_post($post_url, $request);
				}
			}
		}
	}
}

add_action('wp_ajax_displet_user_registration_request', array('DispletRetsIdxUsers', 'register_user_ajax'));
add_action('wp_ajax_nopriv_displet_user_registration_request', array('DispletRetsIdxUsers', 'register_user_ajax'));
add_action('wp_ajax_displetretsidx_user_signon', array('DispletRetsIdxUsers', 'login_user_ajax'));
add_action('wp_ajax_nopriv_displetretsidx_user_signon', array('DispletRetsIdxUsers', 'login_user_ajax'));
add_action('wp_ajax_displet_user_check_request', array('DispletRetsIdxUsers', 'check_user_ajax'));
add_action('wp_ajax_nopriv_displet_user_check_request', array('DispletRetsIdxUsers', 'check_user_ajax'));
add_action('wp_ajax_displet_check_login_request', array('DispletRetsIdxUsers', 'check_login_ajax'));
add_action('wp_ajax_nopriv_displet_check_login_request', array('DispletRetsIdxUsers', 'check_login_ajax'));
add_action('wp_ajax_displet_delete_users_request', array('DispletRetsIdxUsers', 'delete_users'));
add_action('wp_login', array('DispletRetsIdxUsers', 'update_last_login'), 10, 2);
add_action('wp_ajax_displet_update_searches', array('DispletRetsIdxUsers', 'update_viewed_searches'));
add_action('wp_ajax_nopriv_displet_update_searches', array('DispletRetsIdxUsers', 'update_viewed_searches'));
add_action('wp_ajax_displet_update_properties', array('DispletRetsIdxUsers', 'update_viewed_properties'));
add_action('wp_ajax_nopriv_displet_update_properties', array('DispletRetsIdxUsers', 'update_viewed_properties'));
add_action('wp_ajax_displet_save_property_request', array('DispletRetsIdxUsers', 'save_property'));
add_action('wp_ajax_nopriv_displet_save_property_request', array('DispletRetsIdxUsers', 'save_property'));
add_action('wp_ajax_displet_delete_properties_request', array('DispletRetsIdxUsers', 'delete_saved_properties'));
add_action('wp_ajax_nopriv_displet_delete_properties_request', array('DispletRetsIdxUsers', 'delete_saved_properties'));
add_action('wp_ajax_displet_save_search_request', array('DispletRetsIdxUsers', 'save_search'));
add_action('wp_ajax_nopriv_displet_save_search_request', array('DispletRetsIdxUsers', 'save_search'));
add_action('wp_ajax_displet_delete_searches_request', array('DispletRetsIdxUsers', 'delete_saved_searches'));
add_action('wp_ajax_nopriv_displet_delete_searches_request', array('DispletRetsIdxUsers', 'delete_saved_searches'));
add_action('init', array('DispletRetsIdxUsers', 'set_referral_cookie'));
add_action('init', array('DispletRetsIdxUsers', 'add_custom_user_role'));
add_action('admin_menu', array('DispletRetsIdxUsers', 'add_saved_properties_menu'));
add_action('admin_menu', array('DispletRetsIdxUsers', 'add_saved_searches_menu'));
add_action('admin_bar_menu', array('DispletRetsIdxUsers', 'add_search_properties_to_toolbar'), 999);
add_action('admin_bar_menu', array('DispletRetsIdxUsers', 'add_saved_properties_to_toolbar'), 999);
add_action('admin_bar_menu', array('DispletRetsIdxUsers', 'add_saved_searches_to_toolbar'), 999);
add_action('admin_bar_menu', array('DispletRetsIdxUsers', 'clean_up_user_toolbar'), 9999);
add_action('admin_menu', array('DispletRetsIdxUsers', 'clean_up_user_dashboard'), 9999);

?>